(function handleOverrides(data) {
	let overrides = {
		TRADING_LIMIT: data.pair.whatstrat.TRADING_LIMIT,
		BAG_SIZE: data.pair.whatstrat.BAG_SIZE || 'none',
		BAG_VALUE: data.pair.whatstrat.BAG_VALUE || 'none',
	};

	const bid = data.pair.Bid;
	const quoteBalance = data.pair.quoteBalance;
	const mvts = parseFloat(data.pair.whatstrat.MIN_VOLUME_TO_SELL);
	const ABP = parseFloat(data.pair.ABP);
	const keepQuote = parseFloat(data.pair.whatstrat.KEEP_QUOTE);
	const baseReservePct = parseFloat(data.pair.whatstrat.BASE_RESERVE_PCT) / 100 || 0; 
	const notEnoughCandles = typeof data.pair.candlesclose === 'undefined' ? true : data.pair.candlesclose.length < 80;
	const maxSameBase = parseFloat(data.pair.whatstrat.MAX_PAIR_PER_BASE);
	const totalBaseValue = parseFloat(data.pair.whatstrat.TOTAL_BASE_VALUE) - (parseFloat(data.pair.whatstrat.TOTAL_BASE_VALUE) * baseReservePct);
	const tl = parseFloat(data.pair.whatstrat.TRADING_LIMIT);
	const TLRatio = parseFloat(data.pair.whatstrat?.TL_RATIO) || 40;
	const lastOrderType = data.pair?.orders?.[0]?.type
	const updateThreshold = 0.005

	// update only if change exceeds updateThreshold
	function updateIfEnoughDifference(oldValue, newValue) {
		if (typeof oldValue == 'string' && typeof newValue != 'string') {
			return newValue;
		}
		else if (typeof oldValue != 'string' && typeof newValue == 'string') {
			return newValue;
		}
		const minimumDifference = oldValue * updateThreshold
		let difference;
		if (oldValue > newValue) {
			difference = oldValue - newValue;
		} else {
			difference = newValue - oldValue;
		}
		if (difference > minimumDifference) {
			return newValue;
		}
		else {
			return oldValue;
		}
	}
	
	// balance tracking
	function setBagSize() {
		if (overrides.BAG_SIZE !== 'none') {
			if (lastOrderType === 'sell' && (quoteBalance - keepQuote) * bid < mvts) {
				overrides.BAG_SIZE = 'none';
				overrides.BAG_VALUE = 'none'
			}
			else if ((quoteBalance - keepQuote) * ABP > 0) {
				overrides.BAG_SIZE = updateIfEnoughDifference(overrides.BAG_SIZE, (quoteBalance - keepQuote) * ABP)
				overrides.BAG_VALUE = updateIfEnoughDifference(overrides.BAG_VALUE, (quoteBalance - keepQuote) * bid)
			}
		}
		else if ((quoteBalance - keepQuote) * bid < mvts) {
			overrides.BAG_SIZE = 'none';
			overrides.BAG_VALUE = 'none'
		}
		else {
			if ((quoteBalance - keepQuote) * ABP > 0) {
				overrides.BAG_SIZE = updateIfEnoughDifference(overrides.BAG_SIZE, (quoteBalance - keepQuote) * ABP)
				overrides.BAG_VALUE = updateIfEnoughDifference(overrides.BAG_VALUE, (quoteBalance - keepQuote) * bid)
			}
			else {
				overrides.BAG_SIZE = 'none';
				overrides.BAG_VALUE = 'none'
			}
		}
	}

	// set compounding trading limit
	function setTradingLimit() {
		const tl = (totalBaseValue / maxSameBase) / TLRatio;
		if (tl > mvts) {
			overrides.TRADING_LIMIT = updateIfEnoughDifference(overrides.TRADING_LIMIT, tl.toPrecision(5))
			overrides.BUY_ENABLED = true;
		}
		else {
			overrides.TRADING_LIMIT = mvts * 1.1;
			overrides.BUY_ENABLED = true;
		}
	}

	// update pair bag stats
	setBagSize();

	// return safe set of overrides in case critical data is missing
	if (Object.is(totalBaseValue, NaN) || totalBaseValue === 0 || notEnoughCandles) {
		return overrides;
	}

	// set new trading limit after selling a bag
	if (overrides.BAG_SIZE === 'none' && quoteBalance * bid < mvts) {
		setTradingLimit()
	}
	// set trading limit if the current one is obviously wrong
	if (overrides.BAG_SIZE !== 'none' && tl < mvts) {
		setTradingLimit()
	}

	// save overrides to pairVariables
	return overrides;
})(this)