const fs = require("fs");
const minify = require("minify");
const options = {
    js: {
        ecma: 5,
        compress: false,
        mangle: false,
        format: { quote_style: 1 },
        keep_fnames: true,
    },
};

// templates for config.js and autoconfig.json
const defaultConfig = JSON.parse(fs.readFileSync("src/config.js"));
const defaultAutoconfig = JSON.parse(fs.readFileSync("src/compounding_manual-pairs/autoconfig.json"));

// definitions for different config builds
const outputVariations = [
    // auto pairs + compounding
    {
        exchange: "mex_gunthy",
        compounding: true,
        manualPairs: true,
        baseCurrencies: [],
        maxPairPerBase: 3,
        pairExcludes: "",
        pairIncludes: "-",
        fees: 0.1,
    },
    {
        exchange: "okgunbot",
        compounding: true,
        manualPairs: true,
        baseCurrencies: [],
        maxPairPerBase: 3,
        pairExcludes: "",
        pairIncludes: "-",
        fees: 0.1,
    },
    {
        exchange: "beaxy",
        compounding: true,
        manualPairs: true,
        baseCurrencies: [],
        maxPairPerBase: 3,
        pairExcludes: "",
        pairIncludes: "-",
        fees: 0.1,
    },
    {
        exchange: "binance",
        compounding: true,
        manualPairs: true,
        baseCurrencies: [],
        maxPairPerBase: 3,
        pairExcludes: "",
        pairIncludes: "-",
        fees: 0.1,
    },
    {
        exchange: "binanceus",
        compounding: true,
        manualPairs: true,
        baseCurrencies: [],
        maxPairPerBase: 3,
        pairExcludes: "",
        pairIncludes: "-",
        fees: 0.1,
    },
    {
        exchange: "ftx",
        compounding: true,
        manualPairs: true,
        baseCurrencies: [],
        maxPairPerBase: 3,
        pairExcludes: "",
        pairIncludes: "-",
        fees: 0.1,
    },
    {
        exchange: "ftxus",
        compounding: true,
        manualPairs: true,
        baseCurrencies: [],
        maxPairPerBase: 3,
        pairExcludes: "",
        pairIncludes: "-",
        fees: 0.1,
    },
    {
        exchange: "gateio",
        compounding: true,
        manualPairs: true,
        baseCurrencies: [],
        maxPairPerBase: 3,
        pairExcludes: "",
        pairIncludes: "-",
        fees: 0.1,
    },
    {
        exchange: "hitbtc",
        compounding: true,
        manualPairs: true,
        baseCurrencies: [],
        maxPairPerBase: 3,
        pairExcludes: "",
        pairIncludes: "-",
        fees: 0.1,
    },
    {
        exchange: "kucoin",
        compounding: true,
        manualPairs: true,
        baseCurrencies: [],
        maxPairPerBase: 3,
        pairExcludes: "",
        pairIncludes: "-",
        fees: 0.1,
    },
    {
        exchange: "mexc",
        compounding: true,
        manualPairs: true,
        baseCurrencies: [],
        maxPairPerBase: 3,
        pairExcludes: "",
        pairIncludes: "-",
        fees: 0.1,
    },
    {
        exchange: "okex5",
        compounding: true,
        manualPairs: true,
        baseCurrencies: [],
        maxPairPerBase: 3,
        pairExcludes: "",
        pairIncludes: "-",
        fees: 0.1,
    },
    {
        exchange: "txbit",
        compounding: true,
        manualPairs: true,
        baseCurrencies: [],
        maxPairPerBase: 3,
        pairExcludes: "",
        pairIncludes: "-",
        fees: 0.1,
    },
    {
        exchange: "bittrex",
        compounding: true,
        manualPairs: true,
        baseCurrencies: [],
        maxPairPerBase: 3,
        pairExcludes: "",
        pairIncludes: "-",
        fees: 0.1,
    },
    {
        exchange: "bitfinex",
        compounding: true,
        manualPairs: true,
        baseCurrencies: [],
        maxPairPerBase: 3,
        pairExcludes: "",
        pairIncludes: "-",
        fees: 0.1,
    },
    {
        exchange: "cex",
        compounding: true,
        manualPairs: true,
        baseCurrencies: [],
        maxPairPerBase: 3,
        pairExcludes: "",
        pairIncludes: "-",
        fees: 0.1,
    },
    {
        exchange: "gdax",
        compounding: true,
        manualPairs: true,
        baseCurrencies: [],
        maxPairPerBase: 3,
        pairExcludes: "",
        pairIncludes: "-",
        fees: 0.1,
    },
    {
        exchange: "huobi",
        compounding: true,
        manualPairs: true,
        baseCurrencies: [],
        maxPairPerBase: 3,
        pairExcludes: "",
        pairIncludes: "-",
        fees: 0.1,
    },
    {
        exchange: "kraken",
        compounding: true,
        manualPairs: true,
        baseCurrencies: [],
        maxPairPerBase: 3,
        pairExcludes: "",
        pairIncludes: "-",
        fees: 0.1,
    },
    {
        exchange: "poloniex",
        compounding: true,
        manualPairs: true,
        baseCurrencies: [],
        maxPairPerBase: 3,
        pairExcludes: "",
        pairIncludes: "-",
        fees: 0.1,
    }
];

void (async function buildAC() {
    // functions to use in autoconfig.json
    // defaults get assigned while looping through output variations
    let handleOverrides
    let totalBaseBalance

    // for compounding-auto-pairs
    const handleOverrides_default = " " + (await minify("src/compounding_manual-pairs/handleOverrides.js", options)).slice(0, -1);
    const totalBaseBalance_default = " " + (await minify("src/compounding_manual-pairs/totalBaseBalance.js", options)).slice(0, -1);

    // build configs
    outputVariations.forEach((item) => {
        const modifiedConfig = JSON.parse(JSON.stringify(defaultConfig));
        handleOverrides = handleOverrides_default
        totalBaseBalance = totalBaseBalance_default

        const mvtsDefault = "MIN_VOLUME_TO_SELL:base==='USDT'||base==='BUSD'||base==='USDC'||base==='USD'||base==='EUR'?10:base==='BTC'?2e-4:base==='ETH'?.005:base==='BNB'?.05:2e-4"
        const mvtsKraken = "MIN_VOLUME_TO_SELL:base==='USDT'||base==='BUSD'||base==='USDC'||base==='USD'||base==='EUR'?60:base==='BTC'?0.001:base==='ETH'?.005:base==='BNB'?.05:2e-4"
        const compatibleMvts = item.exchange === 'kraken' ? mvtsKraken : mvtsDefault
        const exchangeDir = `build/${item.exchange}`
        const dir = `build/${item.exchange}/`;

        if (!fs.existsSync(exchangeDir)) {
            fs.mkdirSync(exchangeDir);
        }
        if (!fs.existsSync(dir)) {
            fs.mkdirSync(dir);
        }

        const exchangeSettings = {
            key: "dum...",
            secret: "dum...",
            masterkey: "dum...",
            mastersecret: "dum...",
            delay: item.exchange === 'kraken' ? 8 : item.exchange === 'kucoin' ? 8 : 0,
            market: "spot",
            isEncrypted: false,
            includeCoins: "",
            filteredBase: "",
            filteredQuote: "",
            autoPairs: false,
            tick_interval: 30,
            tick_time: false,
            TRADING_FEES: item.fees,
        };

        modifiedConfig.exchanges[item.exchange] = exchangeSettings;
        const modifiedAutoconfigTemplate = JSON.parse(JSON.stringify(defaultAutoconfig))

        // include exchange name in job names
        const renameKeys = (keysMap, obj) =>
            Object.keys(obj).reduce(
                (acc, key) => ({
                    ...acc,
                    ...{ [keysMap[key] || key]: obj[key] }
                }),
                {}
            );
        const newKeys = { "addPairs": `addPairs_${item.exchange}`, "compound": `compound_${item.exchange}`, "removePairs": `removePairs_${item.exchange}` };
        const modifiedAutoconfig = renameKeys(newKeys, modifiedAutoconfigTemplate);

        Object.keys(modifiedAutoconfig).forEach((element) => {
            job = modifiedAutoconfig[element];
            job.pairs.exclude = item.pairExcludes;
            job.pairs.include = item.pairIncludes;
            job.pairs.exchange = item.exchange;

            if (item.compounding) {
                if (element.includes("compound") && item.manualPairs) {
                    job.setPairVariable.newOverrides = handleOverrides;
                    job.overrides.TOTAL_BASE_VALUE = totalBaseBalance;
                }
            }

        });

        //fs.writeFileSync(dir + "/config.js", JSON.stringify(modifiedConfig, null, 4));
        if (item.exchange !== 'binanceus') {
            fs.writeFileSync(dir + "/autoconfig.json", JSON.stringify(modifiedAutoconfig, null, 4).replace(/binance/g, item.exchange).replace(mvtsDefault, compatibleMvts));
        }
        else {
            fs.writeFileSync(dir + "/autoconfig.json", JSON.stringify(modifiedAutoconfig, null, 4).replace(/binanceus/g, 'binance').replace(/binance/g, item.exchange).replace(mvtsDefault, compatibleMvts));
        }
        console.log('config preset saved to:', dir)

    });

    console.log('Done. Get your configs in the build folder.')
})();
